'use strict';

define(function (require) {
    var TaskViewModel = require('taskViewModel');
    var ko = require('knockout');
    var metadataCache = require('metadataCache');
    require('knockout.validation');
    require('jquery');
    var environment = require('environment');

    /**
     * The view model for the task.
     * @param {Object} context Represents the form/view etc. the task is executed for (e.g. the Incident Form).
     * @param {Object} args If you define arguments in the task declaration (_tasks.json) those are exposed on the args object.
     */
    function ViewModel(context, args) {
        var self = this;
        // This should always be included in your custom task to setup the view model according to internal requirements.
        TaskViewModel.call(self, context);

        // To automatically attach the debugger in your browsers developer tools (F12), enable the line below.
        //debugger;

        // Use knockout observables to simplify databinding UI elements to view model. For more information: https://knockoutjs.com/
        // Create backing properties to collect information from the UI so that these can be used later on in the execute function.
        self.text = ko.observable('');
        self.multilineText = ko.observable('Default value...').extend({
            maxLength: 4000
        });
        self.selectedDate = ko.observable();
        // Tip: Have a look in the sample-task-with-ui.html how we can make use of knockout's if binding and 
        // to conditionally show more/less based on values in the data model.
        self.showMore = ko.observable(false);
        
        // The environment object contains useful information like the logged on user
        self.loggedOnUser = environment.loggedOnUser;

        // If you want to show an enum picker, get the source of you're specific enum from the enumStore in the metadataCache.
        // Tip: Debug and look at the metadataCache.enumStore object to find the correct name of your enum.
        self.statusEnumSource = metadataCache.enumStore.IncidentStatusEnum;
        // To make a property required, use knockout validation. For more information: https://github.com/Knockout-Contrib/Knockout-Validation
        self.status = ko.observable().extend({
            required: {
                params: true
            }
        });

        /**
         * Executed before UI template is loaded with the purpose of validating the context (work item or config item)
         * that the task is executed for. Resolve the promise to let WebFront know it's OK to proceed and render the UI
         * template. Reject the promise, passig an object as follows:
         * {
         *  status: 'info'|'warning'|'error',
         *  message: '<describe why it's not possible to execute the task for this workitem/config item>'
         * }
         * @returns {Promise} Deffered's promise object: http://api.jquery.com/Types/#Deferred.
        */
        self.isValid = function () {
            var validLoading = $.Deferred();

            // Return rejected deffered with info level and message if this incident is not suitible for manipulation by this task.
            if (context.formData().Status() === 'IncidentStatusEnum.Closed') {
                return validLoading.reject({
                    status: 'info',
                    message: 'Not possible to change the status of the incident since it is already Closed'
                });
            }

            // Logging to the developer tools console can be useful during development.
            console.log('Sample Task: Validation was executed successful...');

            // Return a resolved promise when validation passed.
            return validLoading.resolve().promise();
        };

        /**
         * Executed once the user presses OK button when the task has a UI template, in the
         * case of no UI template the function is executed emediatly after the view model is initiated.
         * @returns {Promise} Deffered's promise object: http://api.jquery.com/Types/#Deferred
        */
        self.execute = function () {
            // Prepare a Deffered object
            var deffered = $.Deferred();
            // Do changes to description property if available
            if (context.formData().Description) {
                context.formData().Description('User entered: ' + self.text());
            } else {
                console.log('You entered: ' + self.text());
            }
            // Resolve deffered once you've completed the task. If you're running an async process this will make sure the UI is locked until the deffered is resolved.
            deffered.resolve();
            // Return a Deffered's Promise object that is resolved once the task has completed.
            return deffered.promise();
        };
    }

    return ViewModel;
});